@@ -10,8 +10,10 @@ import httpx
 
 from metaerg import registry
 
-DATAFRAME_COLUMNS = 'id genome contig start end strand type inference subsystems descr taxon notes seq antismash ' \
-                    'signal_peptide tmh tmh_topology blast cdd'.split()
+DATAFRAME_COLUMNS = 'id genome contig start end strand type inference subsystems descr taxon notes aa_seq nt_seq ' \
+                    'antismash signal_peptide tmh tmh_topology blast cdd hmm'.split()
+
+RNA_TARGETS = set("rRNA tRNA tmRNA ncRNA retrotransposon".split())
 
 BASE_DIR = Path()
 TEMP_DIR = Path()
@@ -49,7 +51,7 @@ PATH_TO_SIGNALP = None
 PATH_TO_TMHMM = None
 
 def init(contig_file, database_dir, rename_contigs, rename_genomes, min_contig_length, cpus, force, file_extension,
-         translation_table, delimiter, checkm_dir, gtdbtk_dir, tasks, prefix, create_database, download_database,
+         translation_table, delimiter, checkm_dir, gtdbtk_dir, prefix, create_database, download_database,
          install_deps, path_to_signalp, path_to_tmhmm, log_topics=''):
     global BASE_DIR, TEMP_DIR, HTML_DIR, DATABASE_DIR, CHECKM_DIR, GTDBTK_DIR, GENOME_NAME_MAPPING_FILE, MULTI_MODE,\
            RENAME_CONTIGS, RENAME_GENOMES, MIN_CONTIG_LENGTH, FORCE, FILE_EXTENSION, TRANSLATION_TABLE, \
@@ -61,21 +63,7 @@ def init(contig_file, database_dir, rename_contigs, rename_genomes, min_contig_l
     LOG_FILE = Path('log.txt').absolute()
     log('Initializing execution environment with command line arguments...')
 
-    DATABASE_DIR = Path(database_dir).absolute()
-    if not DATABASE_DIR.is_dir():
-        raise Exception("No database dir provided or database dir is not a dir.")
-    TASKS = tasks
-    if download_database:
-        METAERG_MODE = METAERG_MODE_DOWNLOAD_DATABASE
-        log(f'Ready to download databases.')
-        return
-    elif create_database:
-        METAERG_MODE = METAERG_MODE_CREATE_DATABASE
-        if TASKS == 'all':
-            TASKS = 'PVEBRCS'
-        log(f'Ready to create databases from scratch with tasks {TASKS}.')
-        return
-    elif install_deps:
+    if install_deps:
         METAERG_MODE = METAERG_MODE_INSTALL_DEPS
         BIN_DIR = Path(install_deps).absolute()
         if path_to_signalp and Path(path_to_signalp).is_file():
@@ -86,9 +74,23 @@ def init(contig_file, database_dir, rename_contigs, rename_genomes, min_contig_l
             PATH_TO_TMHMM = Path(path_to_tmhmm).absolute()
         else:
             log('Warning: path to tmhmm tarbal not provided or wrong; tmhmm will not be installed.')
-        if not database_dir or not Path(database_dir).is_dir():
-            raise Exception('Need to provide --database_dir to install helper programs.')
-        log(f'Ready to install helper programs at {BIN_DIR} with {DATABASE_DIR}, {PATH_TO_SIGNALP}, {PATH_TO_TMHMM}.')
+        log(f'Ready to install helper programs at {BIN_DIR} with {PATH_TO_SIGNALP}, {PATH_TO_TMHMM}.')
+        return
+
+    DATABASE_DIR = Path(database_dir).absolute()
+    if not DATABASE_DIR.is_dir():
+        raise Exception("No database dir provided or database dir is not a dir.")
+    if download_database:
+        METAERG_MODE = METAERG_MODE_DOWNLOAD_DATABASE
+        log(f'Ready to download databases.')
+        return
+    elif create_database:
+        METAERG_MODE = METAERG_MODE_CREATE_DATABASE
+        if create_database == 'all':
+            TASKS = 'PVEBRCSA'
+        else:
+            TASKS = create_database
+        log(f'Ready to create databases from scratch with tasks {TASKS}.')
         return
     else:
         contig_file = Path(contig_file).absolute()
@@ -213,7 +215,8 @@ def run_external(exec, stdin=None, stdout=subprocess.DEVNULL, stderr=subprocess.
         log(exec)
     result = subprocess.run(exec.split(), stdout=stdout, stdin=stdin, stderr=stderr)
     if result.returncode != 0:
-        raise Exception(f'Error while trying to run "{exec}"')
+        log(f'WARNING: Error while trying to run "{exec}"')
+        print(result.stderr)
 
 
 def download(url: str, file: Path):
