@@ -37,7 +37,7 @@ def _read_results(genome_name, contig_dict, feature_data: pd.DataFrame, result_f
                                    'strand': strand,
                                    'type': 'tRNA',
                                    'inference': 'aragorn',
-                                   'seq': seq,
+                                   'nt_seq': seq,
                                    'descr': f'{trna}-{codon}'}
                         new_features.append(feature)
     feature_data = pd.concat([feature_data, pd.DataFrame(new_features)], ignore_index=True)
