@@ -14,6 +14,7 @@ import ncbi.datasets
 from metaerg import context
 from metaerg.datatypes import fasta
 from metaerg.datatypes.blast import BlastResult, DBentry, TabularBlastParser
+from metaerg.datatypes import ncbi_ftp
 
 DB_DESCRIPTIONS_FILENAME = 'db_descriptions.txt'
 DB_TAXONOMY_FILENAME = 'db_taxonomy.txt'
@@ -30,12 +31,18 @@ IGNORED_FEATURE_TYPES = 'gene pseudogene exon direct_repeat region sequence_feat
 
 
 def _run_programs(genome_name, contig_dict, feature_data: pd.DataFrame, result_files):
-    cds_aa_file = context.spawn_file('cds.faa', genome_name)
     rna_nt_file = context.spawn_file('rna.fna', genome_name)
-    blastn_db = Path(context.DATABASE_DIR, DB_RNA_FILENAME)
+    blastn_result_file = context.spawn_file('blastn', genome_name)
+    if rna_nt_file.exists() and rna_nt_file.stat().st_size:
+        if context.FORCE or not blastn_result_file.exists() or not blastn_result_file.stat().st_size:
+            blastn_db = Path(context.DATABASE_DIR, DB_RNA_FILENAME)
+            context.run_external(f'blastn -db {blastn_db} -query {rna_nt_file} -out {blastn_result_file} -max_target_seqs 10 '
+                                 f'-outfmt 6')
+    else:
+        context.log(f'({genome_name}) Skipping blastn, fasta file with RNA genes missing or empty.')
+
+    cds_aa_file = context.spawn_file('cds.faa', genome_name)
     diamond_db = Path(context.DATABASE_DIR, DB_PROTEINS_FILENAME)
-    context.run_external(f'blastn -db {blastn_db} -query {rna_nt_file} -out {result_files[1]} -max_target_seqs 10 '
-                         f'-outfmt 6')
     context.run_external(f'diamond blastp -d {diamond_db} -q {cds_aa_file} -o {result_files[0]} -f 6 '
                          f'--threads {context.CPUS_PER_GENOME} --fast --max-target-seqs 10')
 
@@ -83,7 +90,9 @@ def _read_results(genome_name, contig_dict, feature_data: pd.DataFrame, result_f
         for blast_result in handle:
             blast_result_count += 1
             process_blast_result(blast_result)
-    with TabularBlastParser(result_files[1], 'BLAST', dbentry_from_string) as handle:
+
+    blastn_result_file = context.spawn_file('blastn', genome_name)
+    with TabularBlastParser(blastn_result_file, 'BLAST', dbentry_from_string) as handle:
         for blast_result in handle:
             blast_result_count += 1
             process_blast_result(blast_result)
@@ -99,7 +108,7 @@ def run_and_read_diamond_blastn():
                            Path(DB_RNA_FILENAME + '.nhr'),
                            Path(DB_DESCRIPTIONS_FILENAME),
                            Path(DB_TAXONOMY_FILENAME)),
-             'result_files': ('diamond', 'blastn'),
+             'result_files': ('diamond',),
              'run': _run_programs,
              'read': _read_results})
 
@@ -355,9 +364,6 @@ def install_prokaryote_database():
     taxon_list = []
     context.log(f'GTDBTK comprises {taxa_count} taxa...')
     with open(gtdbtk_taxonomy_file) as taxonomy_handle, open(descr_db, 'w') as descr_handle:
-        # determine line count for tqdm
-        ftp = FTP('ftp.ncbi.nlm.nih.gov')
-        ftp.login()
         success_count = 0
         genomes_in_cache_count = 0
         count = 0
@@ -377,27 +383,19 @@ def install_prokaryote_database():
             else:
                 try:
                     download_status = ''
-                    ftp.cwd('/genomes/all/')
-                    for acc_part in (accession[0:3], accession[4:7], accession[7:10], accession[10:13]):
-                        ftp.cwd(acc_part)
-                    ftp_dir_list = []
-                    ftp.dir('.', ftp_dir_list.append)
-                    ftp.cwd(ftp_dir_list[0].split()[-1])
-                    ftp_dir_list = []
-                    ftp.dir('.', ftp_dir_list.append)
-                    for l in ftp_dir_list:
-                        filename = l.split()[-1]
-                        if filename.endswith('_protein.faa.gz') and not future_faa_file.exists():
-                            download_status += '*'
-                            success_count += 1
-                            with open(future_faa_file, "wb") as local_handle:
-                                ftp.retrbinary("RETR " + filename, local_handle.write)
-                            taxon['in_local_cache'] = True
-                            genomes_in_cache_count += 1
-                        if filename.endswith('_rna_from_genomic.fna.gz') and not future_rna_file.exists():
-                            download_status += '*'
-                            with open(future_rna_file, "wb") as local_handle:
-                                ftp.retrbinary("RETR " + filename, local_handle.write)
+                    targets = []
+                    if not future_faa_file.exists():
+                        targets.append(('_protein.faa.gz', future_faa_file))
+                    if not future_rna_file.exists():
+                        targets.append(('_rna_from_genomic.fna.gz', future_rna_file))
+                    success = ncbi_ftp.fetch(accession, targets)
+                    if success[0]:
+                        download_status += '*'
+                        success_count += 1
+                        taxon['in_local_cache'] = True
+                        genomes_in_cache_count += 1
+                    if success[1]:
+                        download_status += '*'
                     if not download_status:
                         continue
                 except EOFError:
